/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* global window, document, tizen, console*/

/**
 * Main application module.
 * Handles application life cycle.
 *
 * @module app
 * @namespace app
 */

// make sure that "app" namespace is created
window.app = window.app || {};

// strict mode wrapper function
(function defineApp(app) {
    'use strict';

    /**
     * Stores HTML img element with dark bulb image.
     *
     * @memberof app
     * @private
     * @type {HTMLElement}
     */
    var bulbOffImage = null,

        /**
         * Stores HTML img element with start popup image.
         *
         * @memberof app
         * @private
         * @type {HTMLElement}
         */
        startupPopupImage = null,

        /**
         * Stores HTML div element with close popup button.
         *
         * @memberof app
         * @private
         * @type {HTMLElement}
         */
        closePopupButton = null,

        /**
         * Stores proximity sensor capability key.
         *
         * @memberof app
         * @private
         * @type {string}
         */
        capability = 'http://tizen.org/feature/sensor.proximity',

        /**
         * Interface to access proximity sensor data.
         *
         * @memberof app
         * @private
         * @type {ProximitySensor}
         */
        proximitySensor = null;

    /**
     * Handles success of proximity sensor start method.
     *
     * @memberof app
     * @private
     */
    function onProximitySensorStart() {
        console.log('The proximity sensor started successfully.');
    }

    /**
     * Handles error of proximity sensor start method.
     *
     * @memberof app
     * @private
     * @param {Error} error
     */
    function onProximitySensorError(error) {
        console.error('onProximitySensor error:', error);
    }

    /**
     * Handles proximity state change.
     * Depends on proximity sensor state shows or hides image.
     *
     * @memberof app
     * @private
     * @param {object} sensorData
     */
    function onProximityStateChange(sensorData) {
        bulbOffImage.classList.remove('visible');
        bulbOffImage.classList.remove('hidden');

        if (sensorData.proximityState === 'FAR') {
            bulbOffImage.classList.add('visible');
        } else {
            bulbOffImage.classList.add('hidden');
        }
    }

    /**
     * Starts proximity sensor.
     *
     * @memberof app
     * @private
     */
    function startProximitySensor() {
        try {
            proximitySensor.start(onProximitySensorStart,
                onProximitySensorError);
        } catch (error) {
            console.error('proximitySensor.start error: ', error);
        }
    }

    /**
     * Sets change listener for proximity sensor.
     *
     * @memberof app
     * @private
     */
    function setChangeListener() {
        try {
            proximitySensor =
                tizen.sensorservice.getDefaultSensor('PROXIMITY');
        } catch (error) {
            console.error('setProximitySensorListener getDefaultSensor ' +
                'error: ', error);
        }

        try {
            proximitySensor.setChangeListener(onProximityStateChange);
        } catch (error) {
            console.error('proximitySensor.setChangeListener error: ',
                error);
        }
    }

    /**
     * Checks if proximity sensor is supported. If yes, returns true, false
     * otherwise.
     *
     * @memberof app
     * @private
     * @returns {boolean}
     */
    function isProximitySensorSupported() {
        try {
            return tizen.systeminfo.getCapability(capability);
        } catch (error) {
            console.error('checkProximitySensorCapability error ', error);
            return false;
        }
    }

    /**
     * Handles close popup button click event.
     * Hides startup popup image and close popup button.
     *
     * @memberof app
     * @private
     */
    function onClosePopupButtonClick() {
        startupPopupImage.classList.add('disabled');
        closePopupButton.classList.add('disabled');
    }

    /**
     * Closes the application.
     *
     * @memberof app
     * @public
     */
    app.exit = function exit() {
        try {
            tizen.application.getCurrentApplication().exit();
        } catch (error) {
            console.error('Application exit failed.', error);
        }
    };

    /**
     * Handles hardware key press event.
     * Closes the application if back key was pressed.
     *
     * @memberof app
     * @private
     * @param {Event} event
     */
    function onDeviceHardwareKeyPress(event) {
        if (event.keyName === 'back') {
            app.exit();
        }
    }

    /**
     * Registers event listeners.
     *
     * @memberof app
     * @private
     */
    function bindEvents() {
        document.addEventListener('tizenhwkey', onDeviceHardwareKeyPress);
        closePopupButton.addEventListener('click', onClosePopupButtonClick);
    }

    /**
     * Assigns ui elements.
     *
     * @memberof app
     * @private
     */
    function assignUiElements() {
        bulbOffImage = document.getElementById('bulb-off-image');
        startupPopupImage = document.getElementById('startup-popup-image');
        closePopupButton = document.getElementById('close-popup-button');
    }

    /**
     * Initializes app module. If proximity sensor is supported, launches
     * functions which sets up proximity sensor, assigns ui elements,
     * binds events. If sensor is not supported, shows alert message and
     * closes application.
     *
     * @memberof app
     * @private
     */
    function init() {
        if (isProximitySensorSupported()) {
            assignUiElements();
            bindEvents();
            setChangeListener();
            startProximitySensor();
        } else {
            window.alert('Proximity sensor is not supported on this device');
            app.exit();
        }
    }

    window.addEventListener('load', init);

})(window.app);
